/** @file ccp_scenario.cpp
 *
 * Scenario for verifying the cache coherence protocol.
 *
 * @author Alexander Kamkin (kamkin@ispras.ru)
 */

#include <ccp_scenario.h>

namespace cpptesk {
namespace examples {
namespace ccp {

bool CacheCoherenceScenario::init(int argc, char **argv)
{
    UNUSED_ARG(argc);
    UNUSED_ARG(argv);

    _model.initialize();
    return true;
}

void CacheCoherenceScenario::finish()
{
    _model.finalize();
}

bool CacheCoherenceScenario::read(Ctx &ctx)
{
    unsigned id;

    IBEGIN

    for(IVAR(i) = 0; (unsigned)IVAR(i) < _model.getTransitionNumber(); IVAR(i)++)
    {
        IACTION
        {
            if((id = _model.getProcessorId(IVAR(i))) != -1u)
            {
                _model.trackCoverage(id, Cache::READ);
                _model.read(id);
            }
            YIELD(_model.invariant());
        }
    }

    IEND
}

bool CacheCoherenceScenario::write(Ctx &ctx)
{
    unsigned id;

    IBEGIN

    for(IVAR(i) = 0; (unsigned)IVAR(i) < _model.getTransitionNumber(); IVAR(i)++)
    {
        IACTION
        {
            if((id = _model.getProcessorId(IVAR(i))) != -1u)
            {
                _model.trackCoverage(id, Cache::WRITE);
                _model.write(id);
            }
            YIELD(_model.invariant());
        }
    }

    IEND
}

bool CacheCoherenceScenario::replace(Ctx &ctx)
{
    unsigned id;

    IBEGIN

    for(IVAR(i) = 0; (unsigned)IVAR(i) < _model.getTransitionNumber(); IVAR(i)++)
    {
        IACTION
        {
            if((id = _model.getProcessorId(IVAR(i))) != -1u && _model.pre_replace(id))
            {
                _model.trackCoverage(id, Cache::REPLACE);
                _model.replace(id);
            }
            YIELD(_model.invariant());
        }
    }

    IEND
}

bool CacheCoherenceScenario::reset(Ctx &ctx)
{
    IBEGIN

    IACTION
    {
        _model.reset();
	YIELD(_model.invariant());
    }

    IEND
}

std::string CacheCoherenceScenario::getState()
{
    return _model.getState();
}

}}} // namespace cpptesk::examples::ccp

