/** @file counter_scen.cpp
 * This file contains implementation of the scenario class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <counter_scen.h>

namespace cpptesk {
namespace examples {
namespace counter {

bool CounterScenario::init(int argc, char ** argv)
{
    UNUSED_ARG(argc);
    UNUSED_ARG(argv);

    dut.initialize();
    dut.delay(5);
    initial_state = get_state_counter();
    return true;
}

void CounterScenario::finish()
{
    dut.finalize();
}

bool CounterScenario::scen_nop(Ctx& ctx)
{
    std::string state = get_state_counter();

    IBEGIN
    IACTION
    {
printf("inside of nop\n");
        dut.cycle();
        if(!state.empty() && state != initial_state)
            while(state == get_state_counter()) 
            {
printf("wait for new state\n");
	        dut.cycle();
	        dut.refresh_coverage();
            }
	YIELD(dut.verdict());
    }
    IEND
}

bool CounterScenario::scen_inc(Ctx& ctx)
{
    InputData msg;

    IBEGIN

    IACTION
    {
printf("inside of Counter::inc\n");
//        dut.refresh_coverage();
        if(dut.isIface1Ready())
	{
            msg = InputData();
printf("start Counter::inc\n");
            dut.start(&Counter::inc, dut.iface1, msg, Counter::START_REQUEST_IFACE);
//        dut.cycle();
        }

        YIELD(dut.verdict());
    }
    IEND
}

std::string CounterScenario::get_state_counter()
{
    const std::vector<Process::PointDescriptor> points = dut.getPipelineSlice();

    std::string state;
    std::stringstream out;

    int cycle_counter = 0;

    for ( unsigned int i = 0 ; i < points.size(); i++ )
    {
        Process::PointDescriptor point = points[i];
        if(point.type == START || point.type == CYCLE)
        {
            cycle_counter++;
	    out << "+" << point.file << ":" << point.line << "-";
//	    if(point.type == START) printf("START:%d");
        }
        else if(point.type == GUARD)
        {
            out << "{" << point.file << ":" << point.line << "}";
        }
    }

    out << "{" << cycle_counter << "}";


//    out << "{counter:" << dut.get_value() << "}";
    state = out.str();

    //DEBUG_PRINTF(DEBUG_USER, "get_state_counter(): %s", state.c_str());
    printf("get_state_counter(): %s\n", state.c_str());
	

    return state;
}

CounterScenario::CounterScenario(void) : ScenarioBase()
{
}
CounterScenario::~CounterScenario(void) {}

ExampleScenario::ExampleScenario(void) : CounterScenario()
{
    setup("CounterScenario", &CounterScenario::init, &CounterScenario::finish, &CounterScenario::get_state_counter);

    SCENARIO_METHOD(CounterScenario::scen_nop);
    SCENARIO_METHOD(CounterScenario::scen_inc);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

