/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Mikhail Chupilko (chupilko@ispras.ru)
// SYNOPSIS: Reaction arbiters
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/arbiter/base.hpp>

namespace cpptesk {
namespace hw {

////////////////////////////////////////////////////////////////////////////////
// Two-Level Reaction Arbiter
////////////////////////////////////////////////////////////////////////////////

#define CPPTESK_TWO_LEVEL_ARBITER TwoLevelArbiter

#ifdef HW_SHORT_NAMES
    #define TWO_LEVEL_ARBITER CPPTESK_TWO_LEVEL_ARBITER
#endif // HW_SHORT_NAMES

template <class PrimaryArbiter, class SecondaryArbiter>
  class TwoLevelArbiter : public Arbiter
{
public:
    TwoLevelArbiter() {}
    virtual ~TwoLevelArbiter() {}

    virtual void appendRequest(const Process &process, const Message &message)
    {
        arb1.appendRequest(process, message);
        arb2.appendRequest(process, message);
    }

    virtual bool chooseRequest(const Process &process, const Message &message) const
    {
        if (arb1.chooseRequest(process, message))
            return arb2.chooseRequest(process, message);

        return false;
    }

    virtual void removeRequest(const Process &process, const Message &message)
    {
        arb1.removeRequest(process, message);
        arb2.removeRequest(process, message);
    }
    
    virtual bool isModelBased() const { return arb1.isModelBased() && arb2.isModelBased(); }

protected:
    PrimaryArbiter arb1;
    SecondaryArbiter arb2;

};

}} // namespace cpptesk::hw
